@@ -169,11 +169,13 @@ repo_new_resolve(JSContext *context,
  */
 GJS_NATIVE_CONSTRUCTOR_DECLARE(repo)
 {
-    JSObject *object = NULL;                            \
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(repo)
     Repo *priv;
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(repo);
 
+    (void) argv;
+
     priv = g_slice_new0(Repo);
 
     GJS_INC_COUNTER(repo);
