@@ -1708,11 +1708,13 @@ exports_new_resolve(JSContext *context,
  */
 GJS_NATIVE_CONSTRUCTOR_DECLARE(js_exports)
 {
-    JSObject *object = NULL;                            \
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(js_exports)
     Exports *priv;
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(js_exports);
 
+    (void) argv;
+
     priv = g_slice_new0(Exports);
 
     GJS_INC_COUNTER(dbus_exports);
